"use strict"

function Game (state) {
   var that = this;

   // Functions.
   that.GetPlayer = function () {
      return that.player;
   }
   that.StatExists = function (stat) {
      if (stat in that.stats)
         return true;
      console.error ("Stat '" + name + "' does not exist.");
      return false;
   }
   that.ItemExists = function (item) {
      if (item in that.items)
         return true;
      console.error ("Item '" + name + "' does not exist.");
      return false;
   }
   that.StatExistsNow = function (stat) {
      that.stats[stat] = true;
   }
   that.ItemExistsNow = function (item) {
      that.items[item] = true;
   }
   that.DrawSprites = function (x, y) {
      var sprites = [];
      for (var l in that.layers) {
         var layer = that.layers[l];
         for (var s in layer.sprites) {
            var sprite = layer.sprites[s];
            sprites.push (sprite);
         }
      }
      sprites.sort (function (a, b) {
         return a.z - b.z;
      });
      for (var s in sprites) {
         var sprite = sprites[s];
         sprite.Draw (x, y);
      }
   }
   that.DrawStatus = function (x, y) {
      gImageSets["border"].images[0].Draw (x, y);
      that.player.DrawStatus (x, y);
   }
   that.DrawTime = function (x, y) {
      // Draw the clock...
      var hour = parseInt (Math.floor (that.time - 1.00)) % 12;
      while (hour < 0)
         hour += 12;
      var image = gImageSets["clock"].images[hour];
      image.Draw (x, y, 0.50);

      // ...and the hourglass.
      var images  = gImageSets["hourglass"].images;
      var slices  = images.length;
      var percent = (that.time - gStartTime) / gLoseTime;
      var slice   = Math.floor (percent * (slices));
      if (slice >= slices)
         slice = slices - 1;
      images[slice].Draw (x + 32, y, 0.50);
   }
   that.DrawScene = function () {
      switch (that.state) {
         case "title":
            that.DrawTitle (0, null,
               "Press SPACEBAR to start.");
            break;
         case "lose":
            that.DrawTitle (1, that.gameOverMsg,
               "Press SPACEBAR to play again.");
            break;
         case "win":
            that.DrawTitle (2, that.gameOverMsg,
               "Press SPACEBAR to continue.                            ");
            break;
         case "credits":
            that.DrawTitle (3, that.gameOverMsg,
               "Press SPACEBAR to play again.");
            break;
         case "playing":
            that.DrawPlaying (false);
            break;
         case "landmark":
            //that.landmark.Draw (0, 0);
            gImageSets["border"].images[2].Draw (0, 0);
            var x = gWidth / 2;
            that.DrawText (that.landmark.name, 24, "#cccccc", "center", x, 28);
            that.DrawText (that.landmark.desc, 16, "#cccccc", "center", x, 64);
            //that.DrawText (that.landmark.desc, 16, "#cccccc", "left", 16, 64);
            that.DrawText ("Press SPACEBAR to continue.", 16, "#cccccc",
               "center", x, gHeight - 20);
            break;
         case "store":
            that.store.Draw (0, 0);
            break;
         case "event":
            that.DrawPlaying (true);
            break;
         default:
            console.error ("Unhandled state '" + that.state +
                           "' for DrawScene().");
            break;
      }
   }
   that.SpliceMessages = function () {
      while (that.eventMsgs.length > 0) {
         var goOn = that.eventMsgs[0].goOn;
         that.eventMsgs.splice (0, 1);
         if (goOn !== true)
            break;
      }
   }
   that.DrawPlaying = function (inEvent) {
      // Draw the top half.
      that.DrawSprites (0, 0);
      that.DrawTime    (4, 4);

      // Draw event stuff?
      if (inEvent === true) {
         gImageSets["border"].images[1].Draw (0, 128);

         var y = 150;
         for (var i = 0; i < that.eventMsgs.length; i++) {
            that.eventMsgMax = i + 1;
            var msg = that.eventMsgs[i];
            that.DrawText (msg.message, 16.00, msg.color, "left", 16, y);
            y += 16 + CountChar (msg.message, '\n') * 12;
            if (msg.goOn !== true)
               break;
         }
         if (that.event.done) {
            var msg = "";
            if (that.eventMsgs.length - that.eventMsgMax <= 0)
               msg = "Press SPACEBAR to continue...";
            else
               msg = "(more...)";
            that.DrawText (msg, 16.00, "#cccccc", "center", gWidth / 2,
               gHeight - 20);
         }
         else {
            var x = 16;
            for (var c in that.event.choices) {
               var choice = that.event.choices[c];
               choice.Draw (x, gHeight - 20);
               x += 60;
            }
         }
      }
      else {
         that.map.Draw    (  0, 128);
         that.DrawStatus  (160, 128);
         if (that.extraTime != 0.00) {
            var msg = that.extraTime < 0.00 ? "TIME LOSS" : "BONUS TIME";
            that.DrawText (msg, 32.00, "#000000", "center", 162, 66);
            that.DrawText (msg, 32.00, "#ffffff", "center", 160, 64);
         }
      }
   }
   that.DrawTitle = function (index, text1, text2) {
      gImageSets["title"].images[index].Draw (0, 0);
      if (text1)
         that.DrawText (text1, 16, "#cccccc", "center",
            gWidth / 2, gHeight - 58);
      if (text2)
         that.DrawText (text2, 16, "#cccccc", "center",
            gWidth / 2, gHeight - 20);
   }
   that.DrawText = function (text, size, color, align, x, y) {
      if (text == null)
         return;

      oContext.fillStyle = color;
      oContext.textAlign = align;
      oContext.font = (size * gDrawScale) + "px our-font";

      var texts = text.split (/\n/);
      for (var t in texts) {
         oContext.fillText (texts[t], x * gDrawScale, y * gDrawScale);
         y += size * 0.75;
      }
   }
   that.Tick = function (hours) {
      // If we're in fast-forward mode, increase timer.
      if (that.IsFastForward ())
         hours *= gFastForward;

      // Adjust rain.
      that.rainNext -= hours;
      while (that.rainNext <= 0.00) {
         that.raining = !that.raining;
         that.rainNext += that.NextRain ();
      }

      // Make random events happen!
      if (!that.IsFastForward ()) {
         that.eventNext -= hours;
         while (that.eventNext <= 0.00) {
            that.CauseEvent ();
            that.eventNext += that.NextEvent ();
         }
         if (that.state !== "playing")
            return;
      }

      // Increase game timer. Lose if we run out of time.
      if (that.extraTime <= 0.00) {
         that.time += hours;
         if (that.time - gStartTime >= gLoseTime) {
            that.GameOver (
               "\"This city is dead; time to go home.\n" +
               "Better walk faster next time.\"");
            return;
         }
      }

      // Update all walking layers.
      for (var l in that.layers) {
         var layer = that.layers[l];
         layer.Tick (hours, that.extraTime >= 0.00 ? gWalkSpeed : 0);
      }

      // Move our player forward.
      that.player.Tick (hours, that.extraTime);

      // Force ourselves into landmark mode when we reach them.
      if (that.landmark != null && that.state == "playing")
         that.state = "landmark";

      // Adjust our fast-forward timer.
      if (that.extraTime > 0.00) {
         that.extraTime -= hours;
         if (that.extraTime < 0.00)
            that.extraTime = 0.00;
      }
      if (that.extraTime < 0.00) {
         that.extraTime += hours;
         if (that.extraTime > 0.00)
            that.extraTime = 0.00;
      }
   }
   that.IsRaining = function () {
      return that.raining;
   }
   that.IsStill = function () {
      return (that.state !== "playing");
   }
   that.GameOver = function (msg) {
      that.state = "lose";
      that.gameOverMsg = msg;
      return true;
   }
   that.Victory = function (msg) {
      that.state = "win";
      that.gameOverMsg = msg;
      return true;
   }
   that.KeyDown = function (key) {
      if (key == 87) {
         that.state = "win";
         that.DrawScene ();
         return;
      }
      switch (that.state) {
         case "title":
            if (key == 32)
               that.state = "playing";
            return true;
         case "landmark":
            if (key == 32) {
               if (that.landmark.store) {
                  that.state = "store";
                  that.store = that.landmark.store;
                  that.DrawScene ();
               }
               else
                  that.state = "playing";
               that.landmark = null;
            }
            return true;
         case "win":
            if (key == 32) {
               that.state = "credits";
               that.DrawScene ();
            }
            return true;
         case "lose":
         case "credits":
            if (key == 32) {
               if (gGame && gGame.interval)
                  clearInterval (gGame.interval);
               gGame = new Game ("playing");
            }
            return true;
         case "store":
            that.store.KeyDown (key);
            return true;
         case "playing":
            return true;
         case "event":
            that.event.KeyDown (key);
            return true;
         default:
            console.error ("Unhandled state '" + that.state +
                           "' for KeyDown().");
            return false;
      }
   }
   that.AtLandmark = function (landmark) {
      that.landmark = landmark;
   }
   that.NextRain = function () {
      return RandomFloat (gRainInterval * 0.875, gRainInterval * 1.125);
   }
   that.NextEvent = function () {
      return RandomFloat (gEventInterval * 0.33, gEventInterval * 1.00);
   }
   that.CauseEvent = function () {
      if (that.state === "event")
         return;
      var event = that.map.events[RandomInt (0, that.map.events.length)];
      that.state = "event";
      that.event = event;
      that.eventMsgs = [];
      that.eventMsgMax = 0;
      that.event.done = false;
      if (that.event.selected != null)
         that.event.selected = that.event.choices[0];
      that.EventMessage (event.message, "#cccccc");
      that.event.Run (null);
   }
   that.Gain = function (name, amount, message) {
      that.Change (name, amount, message, "#00ff00");
   }
   that.Lose = function (name, amount, message) {
      that.Change (name, -amount, message, "#ff0000");
   }
   that.Change = function (name, amount, message, color) {
      if (name === "time") {
         that.extraTime += amount;
      }
      else {
         var stat = that.player.GetStat (name);
         if (stat == null) {
            console.error ("Invalid stat '" + name + "' for Change().");
            return;
         }
         stat.Change (amount);
      }
      var msg;
      if (message == null) {
         var verb = (amount < 0) ? "lose" : "gain";
         message = "You " + verb + " " + amount + " " + name + ".";
      }
      that.EventMessage (message, color, true);
   }
   that.IsFastForward = function () {
      return that.extraTime != 0.00;
   }
   that.EventMessage = function (message, color, goOn) {
      that.eventMsgs.push ({
         message: message,
         color:   color,
         goOn:    goOn,
      });
   }
   that.UpdateMusic = function () {
      var nextMusic = "";
      switch (that.state) {
         case "store":
            nextMusic = "Audio/Music/StoreTheme.mp3";
            break;
         case "lose":
            nextMusic = "Audio/Music/DeathTheme.mp3";
            break;
         case "landmark":
            nextMusic = "Audio/Music/LandmarkTheme.mp3";
            break;
         case "win":
         case "credits":
         case "title":
            nextMusic = "Audio/Music/TitleScreen.mp3";
            break;
         default:
            nextMusic = "Audio/Music/WalkingTheme.mp3";
            break;
      }
      if (that.music == nextMusic)
         return;
      that.music  = nextMusic;
      dSource.src = nextMusic;
      dAudio.pause ();
      dAudio.load ();
      dAudio.play ();
   }

   // Initialization.
   that.stats        = [];
   that.items        = [];
   that.layers       = [];
   that.maps         = [];
   that.position     = 0.00;
   that.time         = gStartTime;
   that.rainNext     = that.NextRain ();
   that.eventNext    = that.NextEvent ();
   that.raining      = RandomBool ();
   that.state        = (state == null) ? "title" : state;
   that.gameOverMsg  = "";
   that.landmark     = null;
   that.event        = null;
   that.eventMsgs    = [];
   that.eventMsgMax  = 0;
   that.extraTime    = 0.00;
   that.music        = "";

   // Give ourselves a player.
   new Player (that);

   // Initialize some scene layers.
   new Layer (that, "skybox", {
      stretch:  true,
      noScroll: true,
      zMin:     -10,
      zMax:     -10,
   });
   new Layer (that, "bg", {
      zMin:   -1.00,
      zMax:   -1.00,
      gapMin:   5,
      gapMax:  15,
      xStart: -20,
   });
   new Layer (that, "player", {
      single:   true,
      noScroll: true,
      index:    0,
      xStart:   48,
      hFlip:    true,
      animate:  0.0025,
   });
   new Layer (that, "person", {
      gapMin:   50,
      gapMax:   150,
      zMin:    -0.125,
      zMax:     0.125,
      speedMin: gWalkSpeed * 0.75,
      speedMax: gWalkSpeed * 1.25,
   });
   new Layer (that, "fg", {
      gapMin: 50,
      gapMax: 150,
      zMin:   0.25,
      zMax:   0.25,
   });
   new Layer (that, "rain", {
      noScroll: true,
      zMin:     1.00,
      zMax:     1.00,
      animate:  0.003,
      alwaysAnimate: true,
      onlyIf:   that.IsRaining,
   });

   // Prep for maps / landmarks.
   var imgMaps      = gImageSets["map"].images;
   var imgLandmarks = gImageSets["landmark"].images;

   // Add some maps!
   var map = new Map (that, "SW", imgMaps[0]), l;
   new Landmark (map, "Pioneer Square",            86,  55, imgLandmarks[0],
      "You're at the start of your journey!\n\n" +
      "Welcome to Pioneer Square, also known as\n" +
      "Portland's living room. Opened in 1984, the\n" +
      "square is an open public space located in the\n" +
      "heart of downtown Portland and hosts many\n" +
      "fun events and free shows for all ages\n" +
      "throughout the year.");
   new Store (map.LastLandmark(), [
      { name: "beer",     price: 7, desc: "Willamette Lager" },
      { name: "coffee",   price: 4, desc: "Frappe" },
      { name: "food",     price: 6, desc: "Fresh Salad" },
   ]);
   new Waypoint (map,                              90,  55);
   new Waypoint (map,                              90,  43);
   new Waypoint (map,                             120,  43);
   new Landmark (map, "Saturday Market",          117,  31, imgLandmarks[0],
      "Welcome to the Saturday Market, Portland's\n" +
      "outdoor arts and crafts market. Held every\n" +
      "Saturday and Sunday from the beginning of\n" +
      "March through December, the market is the\n" +
      "largest continuously operated outdoor market\n" +
      "in the United States. Grab a souvenir for a\n" +
      "friend! The market helps support the local\n" +
      "economy and generates an estimated $8 million\n" +
      "in gross sales each year.");
   new Store (map.LastLandmark(), [
      { name: "coffee",   price: 3,  desc: "Cup of Coffee" },
      { name: "food",     price: 5,  desc: "Box of Donuts" },
      { name: "beer",     price: 7,  desc: "Avalanche IPA" },
      { name: "raingear", price: 25, desc: "Rain Poncho" },
   ]);
   new Landmark (map, "Tom McCall Waterfront",    129,  77, imgLandmarks[1],
      "Often cited as being the first instance of\n" +
      "freeway removal, Governor Tom McCall\n" +
      "waterfront is a 36.59 acre park that runs\n" +
      "along the Willamette River. Many activities\n" +
      "happen at the park, including: jogging, walking,\n" +
      "skateboarding, biking, playing in the\n" +
      "fountains, picnics, basketball, fireworks\n" +
      "viewing, and boat watching.");
   new Waypoint (map,                             122, 100);
   new Waypoint (map,                             122, 110);
   new Landmark (map, "Keller Auditorium and Fountain",99,110,imgLandmarks[0],
      "Opened in 1917 and renamed in 1966 after a\n" +
      "generous donation for Richard B Keller, the\n" +
      "auditorium is a very popular performing arts\n" +
      "center home to many performances from the\n" +
      "Portland Opera, the Oregon Ballet Theatre and\n" +
      "beyond. The Keller Fountain Park adjacent to\n" +
      "the performance center, features a award\n" +
      "winning and breath taking concrete fountain\n" +
      "that takes inspiration for the waterfalls\n" +
      "located in the Columbia River Gorge.");
   new Store (map.LastLandmark(), [
      { name: "coffee",   price: 5, desc: "Espresso" },
      { name: "beer",     price: 8, desc: "Kolsch" },
   ]);
   new Waypoint (map,                             102, 110);
   new Waypoint (map,                             102,  88);
   new Landmark (map, "Benson Bubbler",            99,  88, imgLandmarks[0]);
   new Waypoint (map,                             102,  88);
   new Waypoint (map,                             102, 110);
   new Waypoint (map,                              65, 110);
   new Landmark (map, "South Park Blocks",         58, 118, imgLandmarks[1]);
   new Waypoint (map,                              58, 119);
   new Landmark (map, "OHSU Tram",                 22, 119, imgLandmarks[0]);
   new Waypoint (map,                              34, 119);
   new Waypoint (map,                              34, 103);
   new Landmark (map, "Portland State University", 46, 103, imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  3, desc: "Cheap Coffee" },
      { name: "food",     price:  4, desc: "Food?" },
      { name: "beer",     price:  8, desc: "Chocolate Porter" },
      { name: "raingear", price: 25, desc: "Rain Jacket" },
   ]);
   new Waypoint (map,                              49, 103);
   new Waypoint (map,                              49,  79);
   new Landmark (map, "Portland Art Museum",       46,  79, imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  5, desc: "Cappuccino" },
   ]);
   new Landmark (map, "Arlene Schnitzer Concert Hall",71,79,imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  5, desc: "Latte" },
      { name: "beer",     price:  9, desc: "Farmer's Ale" },
   ]);
   new Waypoint (map,                              67,  79);
   new Landmark (map, "Director Park",           67,  56, imgLandmarks[1]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  4, desc: "Latte" },
      { name: "beer",     price:  7, desc: "Avalanche IPA" },
      { name: "food",     price:  6, desc: "Beet Burger" },
   ]);
   new Waypoint (map,                              67,  64);
   new Waypoint (map,                              50,  64);
   new Waypoint (map,                              50,  70);
   new Landmark (map, "Washington Park",           12,  70, imgLandmarks[1]);
   new Landmark (map, "Rose Garden",               12,  63, imgLandmarks[1]);
   new Landmark (map, "Japanese Garden",           12,  56, imgLandmarks[1]);
   new Waypoint (map,                              20,  45);
   new Waypoint (map,                              15,  28);
   new Landmark (map, "Providence Park",           27,  28, imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  4, desc: "Cafe au Lait" },
      { name: "beer",     price:  7, desc: "Bratwurst" },
      { name: "food",     price:  8, desc: "Lumberjack Lager" },
      { name: "raingear", price: 25, desc: "Rain Poncho" },
   ]);
   new Waypoint (map,                              34,  28);
   new Waypoint (map,                              34,  14);
   new Waypoint (map,                              12,  14);
   new Landmark (map, "Pittock Mansion",           12,  15, imgLandmarks[0]);
   new Waypoint (map,                              12,  15);
   new Landmark (map, "Dispensary",                80,  15, imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "beer",     price:  5, desc: "Edibles" },
   ]);
   new Waypoint (map,                              58,  15);
   new Waypoint (map,                              58,  37);
   new Landmark (map, "Foodcart Pod",              67,  37, imgLandmarks[0]);
   new Store (map.LastLandmark(), [
      { name: "coffee",   price:  3, desc: "Cold Brew" },
      { name: "food",     price:  7, desc: "Yakisoba" },
   ]);
   new Waypoint (map,                              86,  55);

   // Assign current map as position our player.
   that.map = map;
   that.player.UseMap (that.map);

   // Add some events to ths map.
   new Event (map,
      "As you are walking you hear a strange music\ncoming from around " +
      "the corner. You decide to\ntake a closer look, and find a band of " +
      "ten\npeople playing some sort of strange,\nelectronic swing music. " +
      "Entranced by their\nrhythms, you lose track of time!",
      function () {
         that.Lose ("time",   0.75,  "You lose 45 minutes!");
         that.Gain ("morale", 10.00);
      });
   new Event (map,
      "A parade of bicyclists blocks your path. But\nwait - these aren't " +
      "just bicyclists... all these\npeople are naked! This must be the " +
      "famous\nNaked Bike Ride!\n\nDo you find a way around or " +
      "stay and watch?", [
         { name: "Watch", func: function () {
            that.EventMessage (
               "You stay and watch the cyclists go by. " +
               "Young,\nold; men, women, and every gender in between -\n" +
               "most are wearing only body paint, fuzzy\nhelmets, or " +
               "the occasional mask.", "#cccccc", true);
            that.Lose ("time", 1.00, "You lost 1 hour!");
            that.Gain ("morale", 25.00);
         } },
         { name: "Avoid", func: function () {
            if (RandomBool ()) {
               that.EventMessage (
                  "You duck down a side street to try to find a\nway around " +
                  "the bicyclists, but you seem to have\ngotten lost.",
                  "#cccccc", true);
               that.Lose ("time", 1.00, "You lost 1 hour!");
            }
            else {
               that.EventMessage (
                  "You duck down a side street and find a gap in\nthe " +
                  "cyclists you can dart through.",
                  "#cccccc", true);
            }
         } },
      ]);

   // Start running!
   var SPF = 1.00 / gFPS;
   that.interval = setInterval (function () {
      that.UpdateMusic ();
      if (gGame != that || that.IsStill ())
         return;
      that.Tick (gHPS / gFPS);
      that.DrawScene ();
   }, SPF * 1000.00);
}
