"use strict"

function Layer (game, name, options) {
   var that = this;

   // Functions.
   that.AddSprite = function () {
      // A lot of conditions in which we shouldn't add a sprite.
      if (that.options.single && that.sprites.length >= 1)
         return false;
      if (that.xNext >= gWidth)
         return false;
      var oldCount = that.sprites.length;
      new Sprite (that);
      return true;
   };
   that.GetImageSet = function () {
      return that.imageSet;
   };
   that.GetIndex = function () {
      return that.nextIndex;
   };
   that.NewIndex = function () {
      var lastIndex = that.nextIndex;
      if (options.index == null)
         that.nextIndex = RandomInt (0, that.imageSet.count);
      else
         that.nextIndex = options.index;
      if (that.nextIndex === lastIndex)
         that.nextIndex = (that.nextIndex + 1) % (that.imageSet.count);
   };
   that.RandomGap = function () {
      return RandomInt (that.options.gapMin, that.options.gapMax);
   };
   that.Update = function () {
      while (that.AddSprite ());
   };
   that.Tick = function (hours, speed) {
      if (that.options.noScroll !== true)
         that.Scroll (hours, speed);
      for (var s in that.sprites) {
         var sprite = that.sprites[s];
         sprite.Tick (hours, speed);
      }
   }
   that.Scroll = function (hours, speed) {
      that.xNext -= (speed + that.speedAvg)
         * Math.pow (2.00, that.zAvg) * hours;
      that.Update ();
      return true;
   }

   // Link to image set. Complain if it doesn't exist.
   that.imageSet = GetImageSet (name);
   if (that.imageSet == null) {
      console.error ("Layer cannot find image set '" + name + "'.");
      return;
   }

   // Link to the game.
   that.game = game;
   game.layers[name] = that;

   // Initialize options.
   if (options == null)
      options = {};
   if (options.single   == null) options.single   = false;
   if (options.stretch  == null) options.stretch  = false;
   if (options.noScroll == null) options.noScroll = false;
   if (options.zMin     == null) options.zMin     = 0.00;
   if (options.zMax     == null) options.zMax     = 0.00;
   if (options.gapMin   == null) options.gapMin   = 0.00;
   if (options.gapMax   == null) options.gapMax   = 0.00;
   if (options.xStart   == null) options.xStart   = 0.00;
   if (options.hFlip    == null) options.hFlip    = false;
   if (options.speedMin == null) options.speedMin = 0.00;
   if (options.speedMax == null) options.speedMax = 0.00;
   if (options.animate  == null) options.animate  = 0.00;
   if (options.alwaysAnimate == null) options.alwaysAnimate = false;
   that.options = options;

   // More initialization.
   that.xNext     = options.xStart + that.RandomGap ();
   that.zAvg      = (options.zMin + options.zMax) / 2.00;
   that.speedAvg  = (options.speedMin + options.speedMax) / 2.00;
   that.sprites   = [];

   // Initialize all elements.
   that.NewIndex ();
   that.Update ();
}
