"use strict"

function Store (landmark, items) {
   var that = this;

   // Link store to our landmark.
   landmark.store = that;
   that.landmark = landmark;

   // Functions.
   that.GetItem = function (name) {
      if (!(name in that.items))
         return null;
      return that.items[name];
   };
   that.LogItems = function () {
      console.log ("Store '" + that.name + "' items:");
      for (var i in that.items) {
         var item = that.items[i];
         console.log ("   $" + item.price + ": " + item.name);
      }
      console.log ("");
   };
   that.Draw = function (x, y) {
      var game = that.landmark.map.game;
      var cursor = gImageSets["cursor"].images[0];
      var xItem = 140;
      var yItem = 28;

      gImageSets["border"].images[3].Draw (0, 0);
      game.player.DrawStatus (160, 128);

      for (var i in that.items) {
         var item = that.items[i];

         if (item == that.selected)
            cursor.Draw (xItem, yItem, 1.00, { vAlign: 1.00 });
         game.DrawText (item.desc, 16.00, "#cccccc", "left",
            xItem + 16, yItem);
         game.DrawText ("$" + item.price.toFixed (0), 16.00, "#cccccc",
            "right", xItem + 150, yItem);
         item.image.Draw (xItem + 150, yItem + 6, 0.33, { vAlign: 1.00, });

         yItem += 20;
      }

      var desc = "";
      if (that.selected == null) {
         cursor.Draw (xItem, 116, 1.00, { vAlign: 1.00 });
         desc = "Leave the store and\n" +
                "continue your journey.";
      }
      else {
         desc = game.player.items[that.selected.name].desc;
      }

      game.DrawText (desc, 16.00, "#cccccc", "center", 84, 180);
      game.DrawText ("Done", 16.00, "#cccccc", "left", xItem + 16, 116);
   };
   that.KeyDown = function (key) {
      var game = that.landmark.map.game;
      switch (key) {
         case 38:
            that.CursorUp ();
            game.DrawScene ();
            break;
         case 40:
            that.CursorDown ();
            game.DrawScene ();
            break;
         case 32:
            // If nothing's selected, we're leaving.
            if (that.selected == null) {
               game.state = "playing";
               game.store = null;
            }
            // Otherwise, we bought an item.
            else {
               if (game.player.Purchase (that.selected, 1))
                  game.DrawScene ();
            }
            break;
      }
   };
   that.CursorUp = function() {
      if (that.selected == null)
         that.selected = that.items[that.items.length - 1];
      else if (that.selected.index == 0)
         that.selected = null;
      else
         that.selected = that.items[that.selected.index - 1];
   }
   that.CursorDown = function() {
      if (that.selected == null)
         that.selected = that.items[0];
      else if (that.selected.index == that.items.length - 1)
         that.selected = null;
      else
         that.selected = that.items[that.selected.index + 1];
   }

   // Initialization.
   that.items = [];
   that.selected = null;

   // Add all our items.
   for (var i in items) {
      var item = items[i];
      new StoreItem (that, item.name, item.price, item.desc);
   }
}
