"use strict"

window.onload = function() {
   // Link our DOM elements.
   dLoading = document.getElementById ("loading");
   dCanvas  = document.getElementById ("canvas");
   dAudio   = document.getElementById ("audio");
   dSource  = document.getElementById ("source");

   // Set up our rendering context.
   oContext = dCanvas.getContext ("2d");
   oContext.imageSmoothingEnabled = false;

   // No more loading!
   dLoading.style.display = "none";

   // Initialize our canvas.
   gWidth  = dCanvas.width  / gDrawScale;
   gHeight = dCanvas.height / gDrawScale;
   ResizeCanvas ();

   // Initialize image sets.
   for (var i in gImageSetNames)
      new ImageSet (gImageSetNames[i]);

   // Keyboard handlers.
   window.addEventListener ('keydown', function (e) {
      gGame.KeyDown (e.which || e.keyCode);
   }, false);

   // Start a new game.
   gGame = new Game ();

   // Draw!
   gGame.DrawScene ();
};

window.onresize = function() {
   ResizeCanvas ();
}

function ResizeCanvas() {
   // Get dimensions of our body.
   var body = document.body,
       html = document.documentElement;
   var width = Math.max (body.scrollWidth, body.offsetWidth, 
       html.clientWidth, html.scrollWidth, html.offsetWidth);
   var height = Math.max (body.scrollHeight, body.offsetHeight, 
       html.clientHeight, html.scrollHeight, html.offsetHeight);

   // Get the limits of our canvas size.
   var ratio = dCanvas.width / dCanvas.height;
   var newWidth  = height * ratio;
   var newHeight = width  / ratio;

   // Cap to limits while maintaining the same aspect ratio.
   if (newWidth > width)   newWidth = width;
   if (newHeight > height) newHeight = height;

   // Set size!
   dCanvas.style.width  = newWidth  + "px";
   dCanvas.style.height = newHeight + "px";
}
